==> python-language-24.2.0-sources.jar: com/oracle/graal/python/nodes/bytecode/KwargsMergeNode.java <==
/*
 * Copyright (c) 2022, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.nodes.bytecode;

import com.oracle.graal.python.builtins.objects.common.HashingStorage;
import com.oracle.graal.python.builtins.objects.dict.PDict;
import com.oracle.graal.python.nodes.PRaiseNode;
import com.oracle.graal.python.nodes.argument.keywords.ConcatDictToStorageNode;
import com.oracle.graal.python.nodes.argument.keywords.NonMappingException;
import com.oracle.graal.python.nodes.argument.keywords.SameDictKeyException;
import com.oracle.truffle.api.dsl.Bind;
import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.GenerateInline;
import com.oracle.truffle.api.dsl.GenerateUncached;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.Frame;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.profiles.InlinedBranchProfile;

@GenerateUncached
@GenerateInline(false) // used in BCI root node
public abstract class KwargsMergeNode extends AbstractKwargsNode {
    public abstract int execute(Frame frame, int stackTop);

    @Specialization
    static int merge(VirtualFrame frame, int initialStackTop,
                    @Bind("this") Node inliningTarget,
                    @Cached ConcatDictToStorageNode concatNode,
                    @Cached PRaiseNode.Lazy raise,
                    @Cached InlinedBranchProfile keywordsError1,
                    @Cached InlinedBranchProfile keywordsError2) {
        int stackTop = initialStackTop;
        Object mapping = frame.getObject(stackTop);
        frame.setObject(stackTop--, null);
        PDict dict = (PDict) frame.getObject(stackTop);
        try {
            HashingStorage resultStorage = concatNode.execute(frame, dict.getDictStorage(), mapping);
            dict.setDictStorage(resultStorage);
        } catch (SameDictKeyException e) {
            keywordsError1.enter(inliningTarget);
            throw handleSameKey(frame, raise.get(inliningTarget), stackTop, e);
        } catch (NonMappingException e) {
            keywordsError2.enter(inliningTarget);
            throw handleNonMapping(frame, raise.get(inliningTarget), stackTop, e);
        }
        return stackTop;
    }

    public static KwargsMergeNode create() {
        return KwargsMergeNodeGen.create();
    }

    public static KwargsMergeNode getUncached() {
        return KwargsMergeNodeGen.getUncached();
    }

}

==> python-language-25.0.2-sources.jar: com/oracle/graal/python/nodes/bytecode/KwargsMergeNode.java <==
/*
 * Copyright (c) 2022, 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.nodes.bytecode;

import com.oracle.graal.python.builtins.objects.common.HashingStorage;
import com.oracle.graal.python.builtins.objects.dict.PDict;
import com.oracle.graal.python.nodes.PRaiseNode;
import com.oracle.graal.python.nodes.argument.keywords.ConcatDictToStorageNode;
import com.oracle.graal.python.nodes.argument.keywords.NonMappingException;
import com.oracle.graal.python.nodes.argument.keywords.SameDictKeyException;
import com.oracle.truffle.api.dsl.Bind;
import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.GenerateInline;
import com.oracle.truffle.api.dsl.GenerateUncached;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.Frame;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.profiles.InlinedBranchProfile;

@GenerateUncached
@GenerateInline(false) // used in BCI root node
public abstract class KwargsMergeNode extends AbstractKwargsNode {
    public abstract int execute(Frame frame, int stackTop);

    @Specialization
    static int merge(VirtualFrame frame, int initialStackTop,
                    @Bind Node inliningTarget,
                    @Cached ConcatDictToStorageNode concatNode,
                    @Cached PRaiseNode raise,
                    @Cached InlinedBranchProfile keywordsError1,
                    @Cached InlinedBranchProfile keywordsError2) {
        int stackTop = initialStackTop;
        Object mapping = frame.getObject(stackTop);
        frame.setObject(stackTop--, null);
        PDict dict = (PDict) frame.getObject(stackTop);
        try {
            HashingStorage resultStorage = concatNode.execute(frame, dict.getDictStorage(), mapping);
            dict.setDictStorage(resultStorage);
        } catch (SameDictKeyException e) {
            keywordsError1.enter(inliningTarget);
            throw handleSameKey(frame, inliningTarget, raise, stackTop, e);
        } catch (NonMappingException e) {
            keywordsError2.enter(inliningTarget);
            throw handleNonMapping(frame, inliningTarget, raise, stackTop, e);
        }
        return stackTop;
    }

    public static KwargsMergeNode create() {
        return KwargsMergeNodeGen.create();
    }

    public static KwargsMergeNode getUncached() {
        return KwargsMergeNodeGen.getUncached();
    }

}
